@@ -1,5 +1,5 @@
 #
-# $Id: ftp.pm,v 1.24.2.1 1998/10/12 11:26:31 aas Exp $
+# $Id: ftp.pm,v 1.29 2001/04/22 05:32:43 gisle Exp $
 
 # Implementation of the ftp protocol (RFC 959). We let the Net::FTP
 # package do all the dirty work.
@@ -19,12 +19,119 @@ require LWP::Protocol;
 
 use strict;
 eval {
+    package LWP::Protocol::MyFTP;
+
     require Net::FTP;
     Net::FTP->require_version(2.00);
+
+    use vars qw(@ISA);
+    @ISA=qw(Net::FTP);
+
+    sub new {
+	my $class = shift;
+	LWP::Debug::trace('()');
+
+	my $self = $class->SUPER::new(@_) || return undef;
+
+	my $mess = $self->message;  # welcome message
+	LWP::Debug::debug($mess);
+	$mess =~ s|\n.*||s; # only first line left
+	$mess =~ s|\s*ready\.?$||;
+	# Make the version number more HTTP like
+	$mess =~ s|\s*\(Version\s*|/| and $mess =~ s|\)$||;
+	${*$self}{myftp_server} = $mess;
+	#$response->header("Server", $mess);
+
+	$self;
+    }
+
+    sub http_server {
+	my $self = shift;
+	${*$self}{myftp_server};
+    }
+
+    sub home {
+	my $self = shift;
+	my $old = ${*$self}{myftp_home};
+	if (@_) {
+	    ${*$self}{myftp_home} = shift;
+	}
+	$old;
+    }
+
+    sub go_home {
+	LWP::Debug::trace('');
+	my $self = shift;
+	$self->cwd(${*$self}{myftp_home});
+    }
+
+    sub request_count {
+	my $self = shift;
+	++${*$self}{myftp_reqcount};
+    }
+
+    sub ping {
+	LWP::Debug::trace('');
+	my $self = shift;
+	return $self->go_home;
+    }
+
 };
 my $init_failed = $@;
 
 
+sub _connect {
+    my($self, $host, $port, $user, $account, $password, $timeout) = @_;
+
+    my $key;
+    my $conn_cache = $self->{ua}{conn_cache};
+    if ($conn_cache) {
+	$key = "$host:$port:$user";
+	$key .= ":$account" if defined($account);
+	if (my $ftp = $conn_cache->withdraw("ftp", $key)) {
+	    if ($ftp->ping) {
+		LWP::Debug::debug('Reusing old connection');
+		# save it again
+		$conn_cache->deposit("ftp", $key, $ftp);
+		return $ftp;
+	    }
+	}
+    }
+
+    # try to make a connection
+    my $ftp = LWP::Protocol::MyFTP->new($host,
+					Port => $port,
+					Timeout => $timeout,
+				       );
+    # XXX Should be some what to pass on 'Passive' (header??)
+    unless ($ftp) {
+	$@ =~ s/^Net::FTP: //;
+	return HTTP::Response->new(&HTTP::Status::RC_INTERNAL_SERVER_ERROR, $@);
+    }
+
+    LWP::Debug::debug("Logging in as $user (password $password)...");
+    unless ($ftp->login($user, $password, $account)) {
+	# Unauthorized.  Let's fake a RC_UNAUTHORIZED response
+	my $mess = scalar($ftp->message);
+	LWP::Debug::debug($mess);
+	$mess =~ s/\n$//;
+	my $res =  HTTP::Response->new(&HTTP::Status::RC_UNAUTHORIZED, $mess);
+	$res->header("Server", $ftp->http_server);
+	$res->header("WWW-Authenticate", qq(Basic Realm="FTP login"));
+	return $res;
+    }
+    LWP::Debug::debug($ftp->message);
+
+    my $home = $ftp->pwd;
+    LWP::Debug::debug("home: '$home'");
+    $ftp->home($home);
+
+    $conn_cache->deposit("ftp", $key, $ftp) if $conn_cache;
+
+    return $ftp;
+}
+
+
 sub request
 {
     my($self, $request, $proxy, $arg, $size, $timeout) = @_;
@@ -77,53 +184,36 @@ sub request
     }
 
     # We allow the account to be specified in the "Account" header
-    my $acct     = $request->header('Account');
+    my $account = $request->header('Account');
 
-    # try to make a connection
-    my $ftp = Net::FTP->new($host, Port => $port);
-    unless ($ftp) {
-       $@ =~ s/^Net::FTP: //;
-       return HTTP::Response->new(&HTTP::Status::RC_INTERNAL_SERVER_ERROR, $@);
-    }
+    my $ftp = $self->_connect($host, $port, $user, $account, $password, $timeout);
+    return $ftp if ref($ftp) eq "HTTP::Response"; # ugh!
 
     # Create an initial response object
-    my $response = HTTP::Response->new(&HTTP::Status::RC_OK,
-				       "Document follows");
+    my $response = HTTP::Response->new(&HTTP::Status::RC_OK, "OK");
+    $response->header(Server => $ftp->http_server);
+    $response->header('Client-Request-Num' => $ftp->request_count);
     $response->request($request);
 
-    my $mess = $ftp->message;  # welcome message
-    LWP::Debug::debug($mess);
-    $mess =~ s|\n.*||s; # only first line left
-    $mess =~ s|\s*ready\.?$||;
-    # Make the version number more HTTP like
-    $mess =~ s|\s*\(Version\s*|/| and $mess =~ s|\)$||;
-    $response->header("Server", $mess);
-
-    $ftp->timeout($timeout) if $timeout;
-
-    LWP::Debug::debug("Logging in as $user (password $password)...");
-    unless ($ftp->login($user, $password, $acct)) {
-	# Unauthorized.  Let's fake a RC_UNAUTHORIZED response
-	my $res =  HTTP::Response->new(&HTTP::Status::RC_UNAUTHORIZED,
-				       scalar($ftp->message));
-	$res->header("WWW-Authenticate", qq(Basic Realm="FTP login"));
-	return $res;
-    }
-    LWP::Debug::debug($ftp->message);
-
     # Get & fix the path
-    my @path =  $url->path_segments;
-    shift(@path);  # There will always be an empty first component
-    pop(@path) while @path && $path[-1] eq '';  # remove empty tailing comps
+    my @path =  grep { length } $url->path_segments;
     my $remote_file = pop(@path);
     $remote_file = '' unless defined $remote_file;
 
-#    my $params = $url->params;
-#    if (defined($params) && $params eq 'type=a') {
-#	$ftp->ascii;
-#    } else {
+    my $type;
+    if (ref $remote_file) {
+	my @params;
+	($remote_file, @params) = @$remote_file;
+	for (@params) {
+	    $type = $_ if s/^type=//;
+	}
+    }
+
+    if ($type && $type eq 'a') {
+	$ftp->ascii;
+    } else {
 	$ftp->binary;
-#    }
+    }
 
     for (@path) {
 	LWP::Debug::debug("CWD $_");
